import 'dart:convert';

import 'package:base_utils/base/system_lifecycle_listener.dart';
import 'package:base_utils/bloc/bloc_providers.dart';
import 'package:base_utils/native_methods.dart';
import 'package:base_utils/share_preference/share_preference_manager.dart';
import 'package:base_utils/ui/animation_widget.dart';
import 'package:base_utils/utils/logging_utils.dart';
import 'package:base_utils/utils/string_utils.dart';
import 'package:base_utils/utils/widget_utils.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

class DeepLink {
  static const MA_GIAM_GIA_CUA_TOI =
      "https://www.sendo.vn/thong-tin-tai-khoan/ma-giam-gia";

  static const THE_LE_DOI_MA_GIAM_GIA =
      "https://www.sendo.vn/su-kien/the-le-doi-ma-giam-gia";

  static const PC3_HOME = "https://www.sendo.vn/tien-ich";
}

class Path {
  static const EVENT = "su-kien";
  static const FLASHSALE = "flash-sale";
  static const PC3 = "tien-ich";
  static const HUONG_DAN = "huong-dan";
}

class Domain {
  static String API_URL;
  static String WEB_API_URL;
  static String MAPI_URL;

  static const JSON_HEADERS = {
    'Content-Type': 'application/json',
    'Accept': 'application/json',
  };

  static void init(String flavor) {
    if (flavor == Router.FLAVOR_DEVELOPMENT) {
      API_URL = 'http://test.sendo.vn/';
      WEB_API_URL = 'http://test.sendo.vn/';
      MAPI_URL = 'http://m-api.test.sendo.vn/';
    } else if (flavor == Router.FLAVOR_PILOT) {
      API_URL = 'https://api-pilot.sendo.vn/';
      WEB_API_URL = 'https://pilot.sendo.vn/';
      MAPI_URL = 'http://mapi-pilot.sendo.vn/';
    } else {
      API_URL = 'https://api.sendo.vn/';
      WEB_API_URL = 'https://sendo.vn/';
      MAPI_URL = 'https://mapi.sendo.vn/';
    }
  }
}

class Redirect {
  static const deeplink = "deeplink";
  static const webview = "webview";
}

class Router implements BaseRouter {
  static const tag = 'Router';

  static const FLAVOR_PRODUCTION = 'production';
  static const FLAVOR_PILOT = 'pilot';
  static const FLAVOR_DEVELOPMENT = 'development';

  static const PROJECT_EVENT = 'event';
  static const PROJECT_FLASHSALE = 'flashsale';
  static const PROJECT_PC3 = 'pc3';
  static const PROJECT_HUONG_DAN = 'huongdan';


  static final Router instance = new Router._internal();

  String _currentProject;
  String _currentFlavor = FLAVOR_PRODUCTION;
  bool _dev = false;

  //account test
  String _userName;
  String _passWord;

  final List<String> _urls = List();

  String initialUrl;

  List<String> get urls => _urls;

  final RouteObserver<Route> routeObserver = new RouteObserver<Route>();

  final _routers = Set<BaseRouter>();

  void addSubRouter(BaseRouter r, [String name]) {
    log('Router.addSubRouter: $r from $name');
    _routers.add(r);
  }

  factory Router() {
    return instance;
  }

  Router._internal() {
    log('Router._internal: called');
  }

  void show({String initialRoute, String url, @required BuildContext context}) {
    assert(context != null);
    assert(isNotEmpty(initialRoute) || isNotEmpty(url));

    if (isEmpty(url) && isEmpty(initialRoute)) return;

    var _len = _urls.length;
    var _curl = _len > 0 ? _urls[_len - 1] : null;

    Map<String, dynamic> data;
    if (isNotEmpty(initialRoute)) {
      data = _readRoute(initialRoute);
      url = isNotEmpty(data['url']) ? data['url'] : url;
    }

    log('$tag: current stack urls= ${_urls.join('\n,')}');
    log('$tag: current url= $_curl');
    log('$tag: next url= ${isNotEmpty(url) ? url : initialRoute}');

    if (isNotEmpty(url) && isNotEmpty(_curl)) {
//      if (url == _curl || url.substring(0,url.length-1) == _curl || _curl.substring(0,url.length-1) == url) {
      if (_curl.replaceAll("/", "") == url.replaceAll("/", "")) {
        log('$tag: is the current.');
        ShakeWidget.shakeScreen(context);
        return;
      } else if (_len > 1) {
        Uri u = Uri.parse(url);
        Uri u1 = Uri.parse(_urls[_len - 2]);
        if (u1 == u) {
          log('$tag: back to previous screen.');
          backOrExit(context, onExit: () => actionCloseScreen());
          return;
        }
      }
    }

    Widget widget =
        getWidget(initialRoute: initialRoute, url: url, context: context);

    if (widget != null) {
      if (widget.key is ValueKey) {
        ValueKey key = widget.key;
        if (key.value == Redirect.webview) {
          log('$tag: navigate to webview.');
//          actionOpenWebView(url: url);
        } else if (key.value == Redirect.deeplink) {
          if (isNotEmpty(initialRoute)) {
            log('$tag: just called from native, navigate to webview.');
//            actionOpenWebView(url: url);
          } else {
            log('$tag: navigate to native screen.');
//            actionDeepLinkSendo(url: url);
          }
        } else {
          log('$tag: navigate to new flutter screen.');
          Navigator.push(
              context, SendoPageRoute(builder: (context) => widget, url: url));
        }
      } else {
        log('$tag: navigate to new flutter screen.');
        Navigator.push(
            context, SendoPageRoute(builder: (context) => widget, url: url));
      }
    } else if (widget is BlocProvider) {
      log('$tag: navigate to new flutter screen.');
      Navigator.push(
          context, SendoPageRoute(builder: (context) => widget, url: url));
    } else {
      log('$tag: navigate to native screen.');
//      actionDeepLinkSendo(url: url);
    }
  }

  @override
  Widget getWidget({BuildContext context, String initialRoute, String url}) {
    Map<String, dynamic> data;
    if (isNotEmpty(initialRoute)) {
      data = _readRoute(initialRoute);
      url = data['url'];
    }

    if (initialUrl == null) initialUrl = url;

    Domain.init(_currentFlavor);
    SharePreferenceManager.initSharePreference();
    SystemLifecycleListener.instance.setHandler();

//    var _len = _urls.length;
//    var _curl = _len > 0 ? _urls[_len - 1] : null;

    if (isNotEmpty(url)) {

      var uri = Uri.parse(url);

      //handle redirect url from other apps.
      var _tmpParam;
      if (uri.queryParameters.containsKey(_tmpParam = 'target_url')) {
        url = uri.queryParameters[_tmpParam];
        log('$tag: target_url=$url');
        uri = Uri.parse(url);
      } else if (uri.queryParameters
          .containsKey(_tmpParam = 'al_applink_data')) {

        try {
          url = json.decode(uri.queryParameters[_tmpParam])["target_url"];
          log('$tag: target_url=$url');
          uri = Uri.parse(url);
        } catch (e) {
          //TODO tracking deeplink kieu moi.
          log(e);
        }
      }

//      String firstPath = getFirstPath(url);
//      log('$tag: firstPath=$firstPath');
//
//      if (isNotEmpty(firstPath)) {
//        switch (firstPath) {
//          case Path.EVENT:
//            _proj = PROJECT_EVENT;
//            break;
//          case Path.FLASHSALE:
//            _proj = PROJECT_FLASHSALE;
//            break;
//          case Path.PC3:
//            _proj = PROJECT_PC3;
//            break;
//          default:
//            break;
//        }
//      }
    }
    /*else {
      _proj = _currentProject;
    }*/

    Widget widget;
    log('Router.getWidget: routers ${_routers.length}');
    for (final router in _routers) {
      widget = router.getWidget(
          context: context, initialRoute: initialRoute, url: url);
      if (widget != null) break;
    }

    log('Router.getWidget: widget ${widget}');

    if (widget == null) {
      widget = Container(
        key: ValueKey(Redirect.deeplink),
      );
    }

//    switch (_proj) {
//      case PROJECT_EVENT:
//        if (isEmpty(url)) url = event.Constants.urlEvent;
//        log('$tag: $url -> mở Event native.');
//        widget = event.Manager.instance.getWidget(url: url, context: context);
//        break;
//      case PROJECT_FLASHSALE:
//        if (isEmpty(url)) url = flashsale.Constants.flashsaleUrl;
//        log('$tag: $url -> mở FlashSale native.');
//        widget =
//            flashsale.Manager.instance.getWidget(url: url, context: context);
//        break;
//      case PROJECT_PC3:
//        if (isEmpty(url)) url = pc3.DeepLink.PC3_HOME;
//        log('$tag: $url -> mở PC3 native.');
//        widget = pc3.Manager.instance.getWidget(url: url, context: context);
//        break;
//      default:
//        log('$tag: $url khong ho tro -> goi deeplink app chinh.');
//        widget = Container(
//          key: ValueKey('deeplink'),
//        );
//        break;
//    }

    if (widget.key != null && widget.key is ValueKey) {
      ValueKey<String> key = widget.key;
      switch (key.value) {
        case Redirect.webview:
          log('$tag: navigate to webview.');
          if (isNotEmpty(initialRoute)) {
            log('$tag: need close Flutter.');
            actionCloseScreen(false);
          }
          actionOpenWebView(url: url);
          break;
        case Redirect.deeplink:
          log('$tag: navigate to native screen.');
          if (isNotEmpty(initialRoute)) {
            log('$tag: need close Flutter.');
            actionCloseScreen(false);
          }
          actionDeepLinkSendo(url: url);
          break;
      }
    }
    return widget;
  }

  ///router: dùng khi mở Flutter từ Native.

  String getDefaultFlavor() {
    return FLAVOR_PRODUCTION;
  }

  Map<String, dynamic> _readRoute(String route) {
    Map<String, dynamic> data;
    if (isEmpty(route)) {
      throw Exception('$tag: route is empty.');
    } else {
      var routeData = parseRoute(route);
      if (routeData != null) {
        _currentProject = routeData['routeName'];
        data = routeData['data'];

        if (isEmpty(_currentProject)) {
          throw Exception('$tag: project route name is empty.');
        }
      }
      if (data != null) {
        _currentFlavor = data['flavor'] ?? _currentFlavor;
        _dev = data["dev"] ?? _dev;
        _userName = data["userName"] ?? _userName;
        _passWord = data["passWord"] ?? _passWord;
      }
    }
    Domain.init(_currentFlavor);
    return data;
  }

  String get currentProject => _currentProject;

  String get currentFlavor => _currentFlavor;

  bool get dev => _dev;

  String get userName => _userName;

  String get passWord => _passWord;

  String get currentUrl => _urls.length > 0 ? _urls[_urls.length - 1] : null;

  void didPop(String url) {
    if (isNotEmpty(url)) {
      log('$tag: didPop $url');
      _urls.remove(url);
    }
  }

  void didPush(String url) {
    if (isNotEmpty(url)) {
      log('$tag: didPush $url');
      _urls.add(url);
    }
  }

  @override
  void initRouter() {}
}

class SendoPageRoute extends CupertinoPageRoute {
  static const tag = 'SendoPageRoute';

  SendoPageRoute({WidgetBuilder builder, this.url}) : super(builder: builder);

  Widget widget;
  final String url;

  @override
  TickerFuture didPush() {
    log('$tag: didPush ${widget.toString()} $url');
    Router.instance.didPush(this.url);
    return super.didPush();
  }

  @override
  Widget buildPage(BuildContext context, Animation<double> animation,
      Animation<double> secondaryAnimation) {
    log('$tag: buildPage ${widget.toString()} $url');

    if (widget == null) {
      widget = ShakeWidget(
        child: super.buildPage(context, animation, secondaryAnimation),
      );
    }

    return widget;
  }

  @override
  void dispose() {
    log('$tag: dispose ${widget.toString()} $url');
    widget = null;
    Router.instance.didPop(this.url);
    super.dispose();
  }
}

abstract class BaseRouter {
  void initRouter();

  Widget getWidget({BuildContext context, String initialRoute, String url});

  @override
  bool operator ==(Object other) =>
      identical(this, other) ||
      other is BaseRouter && runtimeType == other.runtimeType;

  @override
  int get hashCode => 0;
}

class RoutingResult {
  bool openWebView = false;
  bool openDeepLink = false;
  Widget widget;

  RoutingResult.webView() {
    openWebView = true;
    widget = Container();
  }

  RoutingResult.deepLink() {
    openDeepLink = true;
    widget = Container();
  }

  RoutingResult.widget(this.widget);
}
