import 'package:base_utils/utils/string_utils.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:html/dom.dart' as dom;
import 'package:html/parser.dart' as parser;

class BasicHtmlWidget extends StatefulWidget {
  final String html;
  final TextStyle style;
  final TextStyle highlightStyle;
  final TextAlign textAlign;
  final Function(String) onOpenLink;

  const BasicHtmlWidget(this.html,
      {Key key, this.style, this.highlightStyle, this.onOpenLink, this.textAlign = TextAlign.start})
      : super(key: key);

  @override
  _BasicHtmlWidgetState createState() => _BasicHtmlWidgetState();
}

class _BasicHtmlWidgetState extends State<BasicHtmlWidget> {
  dom.Document document;
  TextSpan mainTextSpan;
  final children = List<TextSpan>();

  @override
  void initState() {
    super.initState();
    assert(widget.style != null);
    assert(widget.highlightStyle != null);
    assert(widget.html != null);

    document = parser.parse(widget.html);
    for (dom.Node node in document.body.nodes) {
      if (isEmpty(node.text)) continue;
      if (node is dom.Element && node.localName == 'a') {
        if (node.attributes.containsKey('href')) {
          children.add(TextSpan(
              style: widget.highlightStyle,
              text: node.text,
              recognizer: TapGestureRecognizer()
                ..onTap = () {
                  if (widget.onOpenLink != null) {
                    widget.onOpenLink(node.attributes['href']);
                  }
                }));
          continue;
        }
      }
      children.add(TextSpan(
        style: widget.style,
        text: "${node.text} ",
      ));
    }
  }

  @override
  Widget build(BuildContext context) {
    return RichText(
      textAlign: widget.textAlign,
      text: TextSpan(children: children),
    );
  }
}
